#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/param.h>
#include <assert.h>

#include "config_unix.h"
#include "config_win32.h"
#include "net_udp.h"
#include "debug.h"

#include "util.h"
#include "config.h"
#include "equation.h"
#include "lossfraction.h"

#define LOSS_V2
#undef LOSS_V3
#define QUIET 

int	ppl_cur;
int	ppl_hist[HIST_SIZE];
int	ts_hist[HIST_SIZE];	/* in ms */
int	seq_hist[HIST_SIZE];

u_int32	round;
int	num_loss;
int32	mseq_offs, high_offs;
int32	ppl_start, ppl_end;

int	weights;
int	const_weights;
float	weight[MAX_WEIGHTS];

int	ppl[MAX_WEIGHTS + 1];
/* float   mult[MAX_WEIGHTS + 1]; */
float	new_mult; /* deweighting factor */

int	lost_packet_in_intervall[8];

double	(*lossfraction)();

void init_globals() {
	int i;
	for (i = 0; i < HIST_SIZE; ++i) {
		ppl_hist[i] = 0;
		ts_hist[i] = 0;
		seq_hist[i] = 0;
	}
	for (i = 0; i <= MAX_WEIGHTS; ++i) {
		ppl[i] = 0;
		/* mult[i] = 1.0; */
	}
	mseq_offs = 0;
	high_offs = -1;
	ppl_start = ppl_end = 0;
	ppl_cur = 0;
	weights = 8;
	round = 0;
	num_loss = 0;
	const_weights = 0;
	new_mult = 1.0;

	for(i = 0; i < 7; i++) {
		lost_packet_in_intervall[i]=0;
	}

#ifdef LOSS_V2
	lossfraction = lossfraction_v2;
#else
#ifdef LOSS_V3
	lossfraction = lossfraction_v3;
#else
	lossfraction = lossfraction_v1;
#endif
#endif
}


void set_weights() {
	int i;

	if (const_weights) {
		for (i = 0; i < weights; ++i)
			weight[i] = 1.00;
	} else {
		double delta = 1.0 / ((weights + 1) / 2 + 1);
		for (i = 0; i < weights / 2; ++i)
			weight[i] = 1.00;
		for (i = weights / 2; i < weights; ++i)
			weight[i] = weight[i-1] - delta;
	}
}


void log_loss(int64 arrival) {
	int i;
	double l_fake;

//	ppl[ppl_cur] += mseq_offs - high_offs - 1; /* account for missing packets */

	/* log params */
	ppl_hist[num_loss - 1] = ppl[ppl_cur];
	ts_hist[num_loss - 1] = arrival / MS_TO_US;
	seq_hist[num_loss - 1] = high_offs + 1;

//	high_offs = mseq_offs; /* forget about packets missing inbetween */

	#ifdef LOSS_V2
	/* shift and update mult array and reset new_mult */
	/* no longer necessary because of history faking
	for (i = weights; i > 0; --i) {
		mult[i] = max(mult[i - 1] * new_mult, MIN_MULT);
	}
	mult[0] = 1.0;
	*/
	if (new_mult < 1.0) {
		/* fake history */
#ifndef QUIET
		//printf("%f: history remodeled after deweighting\n",(double) arrival / MICROSEC);
#endif
		l_fake = lossfraction();
		new_mult = 1.0; /* reset deweighting factor */
		for (i = 0; i <= MAX_WEIGHTS; ++i) {
			if (ppl[i] > 0) /* only fake intervals that were already used */
{
//Dragana added
				ppl[i] = (int) (1.0 / l_fake + 0.5);
}
		}
	}
#endif

	ppl_cur = (ppl_cur + 1) % (weights + 1);
	ppl[ppl_cur] = 0;
ppl[ppl_cur] += mseq_offs - high_offs - 1; /* account for missing packets */
high_offs = mseq_offs; /* forget about packets missing inbetween */
}

/*
The mulTFRC equation needs other variables for the equation
n  ... number of TFRC flows
pe ... loss event probability of the cumulative flow
b  ... number of packets acknowledged by one packet - is 1
RTT... round trip time
T  ... initial value for retransmisstion timer - defined as 3
j  ... average number of packets lost in a loss event
*/
double mulTFRC_equation(double n, double pe, double b, double RTT, double T, double j) {
//	RTT = (double) RTT/MILLISEC;
//	T = (double) T/MICROSEC;
	double j1 = j;
if(n<12)j1=n-n*pow((n-1)/n,j1);
	if (j1 > n)
		j1 = ceil(n);
//printf("j ref: %f \n", j1);
	double a = sqrt(pe * b * j1 * (24 * n * n + pe * b * j1 * (n * n - 4  * n * j1 + 4 * j1 * j1)));
	double x = (j1 * pe * b * (2 * j1 - n) + a) / (6 * n * n * pe);
	double w = n * x / (2 * b ) * (1 + 3 * n / j1);
	double z = T * (1 + 32 * pe * pe) / (1 - pe);
	double q1 = j * n / w;
	if (q1 > n)
		q1 = n;
	double q;
	if ((q1 * z / (x * RTT)) >= n) {
		q = n;
	}
	else {
		q = q1 * z / (x * RTT);
	}
//printf("RTT: %f, T: %f j:%f pe: %f x: %f w: %f z: %f q1: %f q: %f \n",RTT,T,j,pe,x,w,z/T,q1,q);
	return (((1 - q / n) / (pe * x * RTT) + q / (z * (1 - pe)))*PKT_SIZE);
}

double weight_lost_packets() {
	double weights[8] = {1.0, 1.0, 1.0, 1.0, 0.8, 0.6, 0.4, 0.2};
	double sum = 0;
	double sum_weights = 0;
	double avg_lost_packets = 0;
	int i;
	for (i = 0; i < 8; i++) {
		if (lost_packet_in_intervall[i]>0){
			sum = sum + ( weights[i] * lost_packet_in_intervall[i] ); 
			sum_weights = sum_weights + weights[i];	
		}
	}

	/* also I_0 ?*/
	avg_lost_packets = sum / sum_weights;
	return avg_lost_packets;
}

int update_history(u_int32 mseq, int64 arrival, double rtt) {
	static int64 lastloss = 0;
	int i;
	mseq_offs = mseq;





static int32 high_offs_retrans=0;
static int first=0;
static int32 losscount=0; 
        

if ((mseq_offs > high_offs+1) && first==0){high_offs_retrans=high_offs+1;first=1;}
if (mseq_offs < high_offs && first==1){high_offs=high_offs_retrans;first=0;}
if (mseq_offs < high_offs && first==0){high_offs=mseq_offs;}

	if ((mseq_offs > high_offs + 1)) { /* new packet */



losscount+=mseq_offs - high_offs+1;


		if ((arrival - lastloss > rtt * MS_TO_US)) { /* new loss event */





			++num_loss;
			if (num_loss >= HIST_SIZE) {
				fprintf(stderr, "loss history buffer overflow\n");
				return 0;
			}

                        /* shift lost packets */
		        for(i = 6; i >=0; i--) {
                                lost_packet_in_intervall[i+1]=lost_packet_in_intervall[i];
                        }

				lost_packet_in_intervall[0] = mseq_offs-high_offs -1;


                
			log_loss(arrival);
			lastloss = arrival;
		
		} else { /* new lost packet in old round trip time */

			lost_packet_in_intervall[0] += mseq_offs-high_offs - 1;
			ppl[ppl_cur] += mseq_offs-high_offs - 1;
			high_offs = mseq_offs;

		}
	}
	if (mseq_offs == high_offs + 1) {
		/* packet in order */
		high_offs = mseq_offs;
	}

	++ppl[ppl_cur]; /* add packet to last interval */

	return 1;
}




u_int32 calc_tzero(double rtt) {
        static int t_srtt_ = 8;
        static int t_rttvar_ = 0;
        double t_rtxcur_;
        int t_rtt_;


t_rtt_ = (int) (rtt / MILLISEC / tcp_tick_ + 0.5);

        if (t_rtt_ < 1)
                t_rtt_ = 1;

        /*
          srtt has 3 bits to the right of the binary point
          rttvar has 2
          */

        if (t_srtt_ != 0) {
                register short delta;
                delta = t_rtt_ - (t_srtt_ >> T_SRTT_BITS);
                if ((t_srtt_ += delta) <= 0)
                        t_srtt_ = 1;
                if (delta < 0)
                        delta = -delta;
                delta -= (t_rttvar_ >> T_RTTVAR_BITS);

                if ((t_rttvar_ += delta) <= 0)
                        t_rttvar_ = 1;

        } else {
                t_srtt_ = t_rtt_ << T_SRTT_BITS;
                t_rttvar_ = t_rtt_ << (T_RTTVAR_BITS-1);
        }

        /*
          Current retransmit value is (unscaled) smoothed round trip estimate
          plus 4 times (unscaled) rttvar.
          */

/*        t_rtxcur_ = (((t_rttvar_ << (2 + (T_SRTT_BITS - T_RTTVAR_BITS))) +
                      t_srtt_)  >> T_SRTT_BITS ) * tcp_tick_;
        if (t_rtxcur_ < 2 * tcp_tick_)
                t_rtxcur_ = 2 * tcp_tick_;
*/




t_rtxcur_ = (((t_rttvar_ << (2 + (T_SRTT_BITS - T_RTTVAR_BITS))) +
                      t_srtt_)  >> T_SRTT_BITS ) * 0.1;
        if (t_rtxcur_ < 0.2)
                t_rtxcur_ = 0.2;

        return t_rtxcur_ * MILLISEC;
}





int main(int argc, char *argv[]) {
	char *addr = NULL;
	int  port = -1;
	int  ttl = 16;

	char send_buffer[REP_SIZE];
	char recv_buffer[PKT_SIZE];
	struct pkt_dtg *packet = (struct pkt_dtg *)recv_buffer;
	struct rep_dtg *rep    = (struct rep_dtg *)send_buffer;

	socket_udp *send_socket;
	socket_udp *recv_socket;

	int64   sim_start = 0, sim_end = 0;
	int64   arrival = 0, last_report = 0, last_flush = 0;

	u_int32 rtt_cur = 0;
	u_int32 rtt_cur_old = 0;			/* rtt_cur of previous packet */
	u_int32 tzero = 0;
	double  rtt_ema = 0.0;
	double  l_act = 0.0;
	double  b_act = 0, b_exp = 0, b_rep = 0;
	/* double  b_exp_max = 0; */

	double mulTFRC = 1;				/* number of TFRC connections */
	double rto;
	double j;

	int	duration = 0;				/* quit after 'duration' seconds */
	int	datalen = 0, data_per_report = 0;
	struct	timeval max_wait;
	
	//int	nready;
	u_int32	mode = M_INITRTT;
	/* int	num_rttinc = 0; */                      /* number of consecutive incs in RTT */
	int	packets = 0;
	u_int32	mseq;
	u_int32	expected_mseq = 0;			/* expected sequence number of next packet */

	u_int32 last_reported_mseq;
	int second_ack_sent = 0;
	char received_data[DATA_SIZE];

	int data_size;					/* size of data in packet*/
static int64 last_change;
	FILE *file_pointer;

	int end_receiver = 0;				/* variable to check if end_flag has been received */
	int recvbuf = 0;				/* size of receiving buffer in bytes */


	max_wait.tv_sec = (long) MAX_WAIT;
	max_wait.tv_usec = ((long) (MAX_WAIT * MICROSEC)) % MICROSEC;

/*	int64 max_wait_for_packet = MAX_WAIT * 1000000;*/
/*	int64 actual = get_time();*/

	int	log_nr = 1;
//	char	log_name[100];
//	FILE	*h_file;

	int32 i;

	/* code to simulate lost packets */
	int	sim_error = 0;
	/**/

	init_globals();

	if (argc < 3) {
		fprintf(stderr, "usage: %s address port [ttl] [-t duration] [-n lognr] [-w num. weights] [-m mulTFRC]\n", argv[0]);
		exit(1);
	} else {
		addr = argv[1];
		port = atoi(argv[2]);
		for (i = 3; i < argc; ++i) {
			if (argv[i][0] != '-') {
				ttl = atoi(argv[i]);
			} else {
				switch (argv[i][1]) {
				case 'n' : if (argc > i + 1) log_nr = atoi(argv[++i]); break;
				case 't' : if (argc > i + 1) duration = atoi(argv[++i]); break;
				case 'w' : if (argc > i + 1) weights = atoi(argv[++i]); break;
				case 'm' : if (argc > i + 1) mulTFRC = atof(argv[++i]); break;
				case 'c' : const_weights = 1; break;
				default : fprintf(stderr, "unknown option: %s\n", argv[i]); exit(1); break;
				}
			}
		}
	}
	set_weights();

#ifndef QUIET
	printf("started ");
	for (i = 0; i < argc; ++i)
		printf("%s ", argv[i]);
	printf("\n");
#endif

	udp_fd_zero();
	send_socket = udp_init(addr, port+1, ttl); /* initiate socket for sending the receiver report, stated port number+1*/
	recv_socket = udp_init(addr, port  , ttl); /* initiate socket for receiving original packets */
	if (send_socket == NULL || recv_socket == NULL) {
		perror("open socket");
		exit(1);
	}

	recvbuf = udp_getrecvbuf(recv_socket);


	init_time();
	arrival = get_time();
	sim_start = get_time() + 1; /* init sim_start */
	if (duration > 0)
		sim_end = sim_start + duration * MICROSEC;
	sim_error=1;

	printf("Simulating %f TFRC connections\n", mulTFRC);

/*	max_wait.tv_usec = 10000;*/



	while (end_receiver==0){//((sim_end == 0 || get_time() < sim_end) && (end_receiver==0)) { /* receiving loop */ 
		udp_fd_set(recv_socket);               
		/*max_wait.tv_sec = (sim_end - get_time()) / MICROSEC + 1;
		max_wait.tv_usec = 0;*/
		//if ((nready = udp_select((duration > 0) ? &max_wait : NULL)) <= 0) {
		/*if ((nready = udp_select(&max_wait)) <= 0) {
			if (nready == 0)
//				printf("Timeout\n");
				break;
			if (errno == EINTR)
				continue;
			else
				perror("select_error");
		}*/
		/*actual = get_time();
		if ((actual/MICROSEC) > ((arrival/MICROSEC) + max_wait_for_packet)) {
			printf("Timeout\n");
			break;			
		}
		
		if ((nready = udp_select(&max_wait)) <= 0) {
			continue;		
		}*/
		if (udp_fd_isset(recv_socket)) {
			//if(datalen = udp_recv(recv_socket, recv_buffer, PKT_SIZE)) {
			datalen = udp_recv(recv_socket, recv_buffer, PKT_SIZE);
			data_per_report += datalen;

			++packets;
			arrival = get_time();
			mseq    = ntohl(packet->mseq); /* get sequence number of packet */
			b_act   = ntohl(packet->bitrate);
			rtt_cur = ntohl(packet->rtt);


			rto	= ntohl(packet->rto);

			data_size = ntohl(packet->data_size);
			//if (ntohl(packet->mulTFRC)>0 && (ntohl(packet->mulTFRC)/100)!=mulTFRC) printf("mulTRFC parameter is %f\n", (double)ntohl(packet->mulTFRC)/100);
			//if(ntohl(packet->mulTFRC)>0)mulTFRC=(double)ntohl(packet->mulTFRC)/100;

			/* rtt_ema = ntohl(packet->rtt); */
			/* if (mode != M_NO_INCREASE) */
			mode = ntohl(packet->mode); /* wait until RTT decreases -> M_NORMAL */



static int red=0;


{





			if (rtt_ema > 0.0)
				rtt_ema = (1 - RTT_DECAY) * rtt_ema + RTT_DECAY * rtt_cur;
			else
				rtt_ema = rtt_cur;


#ifdef NOTDEF
			/* analyzation of inc RTTs no longer needed with inter packet space modulation */
			if (rtt_cur > rtt_cur_old && mode == M_NORMAL) {
				++num_rttinc;
				if (num_rttinc >= BUF_INDICATOR) {
					mode = M_NO_INCREASE;
					mni_ts = arrival;
					b_exp_max = b_exp;
					/* printf("%f: receiver in M_NO_INCREASE mode\n", (double) arrival / MICROSEC); */
				}
			} else if (rtt_cur < rtt_cur_old) {
				if (mode == M_NO_INCREASE) {
					mode = M_NORMAL;
					/* printf("%f: receiver back to M_NORMAL mode\n", (double) arrival / MICROSEC); */
				}
				num_rttinc = 0;
			}
			/* timeout M_NO_INCREASE after 10 RTTs, XXX not very elegant */
			if (mode == M_NO_INCREASE && (arrival - mni_ts) / MS_TO_US > 10 * rtt_ema) {
				mode = M_NORMAL;
				num_rttinc = 0;
				/* printf("%f: receiver back to M_NORMAL mode (timeout)\n", (double) arrival / MICROSEC); */
			}
			#endif

			//tzero = max(4 * max(rtt_ema, rtt_cur), 500);
//Dragana pocetak
tzero=max(calc_tzero(rtt_ema), 200);
//Dragana kraj
			rtt_cur_old = rtt_cur;

			/* adjust round number only if sender has responded to the first loss event
			        * and kicked out of SLOWSTART
			        */
			if (!(mode == M_SLOWSTART && num_loss > 0))
				round = ntohl(packet->round);

u_int32 sendpkt=ntohl(packet->send_pkt);
			if (update_history(sendpkt, arrival, rtt_cur) == 0)
				break; /* -> error */

			int ret;

			b_rep = data_per_report / ((double) (arrival - last_report) / MICROSEC);
			data_per_report = 0;

			/* flush every 500 ms */
			if (arrival - last_flush >= 500000) {
			//	fflush(h_file);
				last_flush = arrival;
			}

			/* calculate expected bitrate */
			if (num_loss > 0) {
				l_act = lossfraction();
				if(mulTFRC == 1) {
					b_exp = bitrate_eq(l_act, rtt_ema, tzero);
				}
				else {
					j = weight_lost_packets();
//					b_exp = mulTFRC_equation(mulTFRC, l_act, 1, (double)rtt_ema / MILLISEC, rto / MICROSEC, j);
double rateC = mulTFRC_equation(mulTFRC, l_act, 1, (double)rtt_ema / MILLISEC, (double)tzero / MILLISEC, j);

if(rateC>b_exp ){
 double mult = (get_time()-last_change)/(rtt_ema*1000 ) ;
        if (mult > 2) mult = 2 ;

        b_exp  = b_exp  + (PKT_SIZE/((double)rtt_ema / MILLISEC))*mult ;

b_exp=(b_exp>rateC)?rateC:b_exp;
}else{b_exp=rateC;}
last_change=get_time();

				}
			} else {
				l_act = 0.0;
				b_exp = 0.0;
			}

#ifdef NOTDEF
			/* not used, see above */
			/* only increase above b_exp_max if not in M_NO_INCREASE mode */
			if (mode == M_NO_INCREASE && b_exp > b_exp_max) {
				b_exp = b_exp_max;
				if (b_exp - b_exp_max > 10)
					printf("%f: M_NO_INCREASE mode, bitrate inc. prevented (%2.1f / %2.1f)\n", (double) arrival / MICROSEC, b_exp, b_exp_max);
			}
#endif
				
			/* report lossrate etc. */
			if (mseq < expected_mseq) { /* retransmission */
				if(packet->start_flag==1) {
					printf("New StartFlag received, new Filepointer created\n");
					fclose(file_pointer);
					memcpy(received_data, packet->data, data_size);
					for(i=data_size; i<DATA_SIZE; i++) {
                                                received_data[i]=NULL;
                                        }
					file_pointer=fopen(received_data,"wb");
					/*printf("Filelength %i Datastream to file %s\n",data_size, received_data);*/
					second_ack_sent = 0;
					rep->mseq = packet->mseq;
					expected_mseq = mseq + 1;
					if (ntohl(packet->mulTFRC)>0 &&( ((double)ntohl(packet->mulTFRC))/100)!=mulTFRC )

 printf("mulTRFC parameter is %f\n", ((double)ntohl(packet->mulTFRC))/100);
		                        if(ntohl(packet->mulTFRC)>0)mulTFRC=((double)ntohl(packet->mulTFRC))/100;
//printf("mulTFRC after changeor npt: %f \n", mulTFRC);
				}
				else {
					//printf("Retransmission of packet %lu \n", mseq);
					rep->mseq = htonl((u_int32)expected_mseq - 1);
					second_ack_sent++;
				}
			}
			if (mseq == expected_mseq) { /* new in order packet */
				rep->mseq = packet->mseq;
				second_ack_sent = 0;

				if(packet->start_flag==1) {
					printf("StartFlag received\n");
					//filename  = (char*) malloc (sizeof(char)*data_size);
				//	strncpy(received_data, packet->data, data_size);
memcpy(received_data, packet->data, data_size);


					for(i=data_size; i<DATA_SIZE; i++) {
						received_data[i]=NULL;
					}
					file_pointer=fopen(received_data,"wb");
					/*strcpy(filename, received_data);
					if(strncmp(filename, received_data, data_size) == 0) {
						printf("here\n");
					}
					printf("Filelength %i Datastream to file %s\n",data_size, filename);*/

					if (ntohl(packet->mulTFRC)>0 &&( ((double)ntohl(packet->mulTFRC))/100)!=mulTFRC )
 printf("mulTRFC parameter is %f\n", ((double)ntohl(packet->mulTFRC))/100);

		                        if(ntohl(packet->mulTFRC)>0)mulTFRC=((double)ntohl(packet->mulTFRC))/100;
//printf("mulTFRC after changeor npt: %f\n", mulTFRC);
				}
				else {
				//	strncpy(received_data, packet->data, data_size);
memcpy(received_data, packet->data, data_size);


					/* manual char completion */
					for(i=data_size; i<DATA_SIZE; i++) {
						received_data[i]=NULL;
					}
				//	fprintf(file_pointer, "%s", received_data); /* speed test */ /* reinitialize */
//fwrite( received_data, sizeof( received_data[0]),  sizeof(received_data)/sizeof( received_data[0]),file_pointer);
fwrite( received_data, sizeof(received_data[0]), data_size,file_pointer);

						}
				if(packet->end_flag==1) {
					printf("EndFlag received\n");
					end_receiver=1;
				}
				expected_mseq++;
			}
				
			if (mseq > expected_mseq) { /* missing packet */
				last_reported_mseq = expected_mseq-1;
				rep->mseq = htonl((u_int32) last_reported_mseq);
				second_ack_sent++; /* send only one duplicate ack*/
			}

			rep->dseq     = packet->dseq;
			rep->ts       = packet->ts;
			rep->b_rep    = htonl((u_int32) b_rep);
			rep->b_exp    = htonl(b_exp);/*(u_int32)*/
			rep->rto	= htonl(tzero);
			rep->timestamp 	= packet->timestamp;
			rep->ignore_packet = htonl(0);
				
			if(second_ack_sent >= 2) {
				rep->ignore_packet = htonl(1);
				rep->mseq = htonl(0);
			}
			
			rep->recvbuf = htonl(recvbuf);

			while ((ret = udp_send(send_socket, (char *) rep, REP_SIZE)) < 0) {
				fprintf(stderr, "%3.2f ", (float) get_time() / MICROSEC);
				perror("udp_send");
			}

			last_report = arrival;
			packets = 0;
			/* log relevant variables */
			/* 1 time, 2 seqno, 3 round, 4 loss, 5 bact, 6 bexp, 7 brep, 8 rtt_ema, 9 rtt_cur, 10 tzero, ... */
			/*fprintf(h_file, "%f %li %lu %f %1.0f %1.0f %1.0f %f %lu %lu",(float) (arrival - sim_start) / MICROSEC, mseq, round, l_act, b_act, b_exp, b_rep, rtt_ema, rtt_cur, tzero);*/ /* speed test */
			/* 11 ppl_start, 12 ppl_end */
			/*fprintf(h_file, " %li %li", ppl_start, ppl_end);
			//for (i = 0; i <= weights; ++i)
		//		fprintf(h_file, " %i", ppl[(ppl_cur - i + weights + 1) % (weights + 1)]);*/ /* speed test */ 
 			/* fprintf(h_file, "\n"); */
			/*fprintf(h_file, " %f\n", new_mult);*/ /* speed test */

//Dragana
}
red++;


		} /* end packet processing */
	} /* end receiving loop */

	printf("%f: receiver finished\n", (float) get_time() / MICROSEC);
//	fclose(h_file);
	if( packets > 0 )
		fclose(file_pointer); /* does not work, creates segmentation fault if no filepointer was created */

//	sprintf(log_name, "losshist%i.log", log_nr);
//	h_file = fopen(log_name, "w");
//	for (i = 0; i < num_loss; ++i)
//		fprintf(h_file, "%f %i %i\n", (float) ts_hist[i] / MILLISEC, ppl_hist[i], seq_hist[i]);
//	fclose(h_file);

//	sprintf(log_name, "weights%i.log", log_nr);
//	h_file = fopen(log_name, "w");
//	for (i = 0; i < weights; ++i)
//		fprintf(h_file, "%f ", weight[i]);
//	fprintf(h_file, "\n");
//	fclose(h_file);
	return 0;
}
