#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/shm.h>
#include <sys/time.h>
#include <sys/select.h>
#include <sys/param.h>
#include <assert.h>
#include <math.h>

#include "config_unix.h"
#include "config_win32.h"
#include "net_udp.h"
#include "debug.h"
#include "config.h"
#include "util.h"

typedef struct list_node {
	int64 time;
	u_int32 packet_id;
	struct list_node *next;
} llnode;

struct shared {
        u_int32 send_pkt;
	u_int32	packets_sent;
	u_int32	reports_recv;
	u_int32	rtt_cur;
	double	rtt;
	double	sqrt_rtt;
	double	b_rep;
	double	b_act;
	double	b_exp;
	double	min_bitrate;
	double	max_bitrate;
	double	max_increase;
	double	rate_delta;
	int64	adj_ts;
	int64	rep_ts;
	u_int32	round;
	int	mode;
	int	force_n_mode;

	u_int32	packets_to_be_sent;
	int	receiver_finished;
	u_int32	expected_mseq;
	int	no_timeout;
	int	recvbuf;
	int	size_of_packet;
	int64	recv_window;
	double	rto;
};

int shm_init(struct shared *info, double minb, double maxb) {
        info->send_pkt	=0;
	info->packets_sent	= 0;
	info->reports_recv	= 0;
	info->rtt_cur		= 0;
	info->rtt		= 0.0;
	info->sqrt_rtt		= 0.0;
	info->b_rep		= 0.0;
	info->b_exp		= PKT_SIZE * 2;
	info->b_act		= info->b_exp;
	info->min_bitrate	= minb;
	info->max_bitrate	= maxb;
	info->max_increase	= info->b_exp;
	info->rate_delta	= 0.0;
	info->adj_ts		= 0;
	info->rep_ts		= 0;
	info->round		= 0;
	info->mode		= M_INITRTT;
	info->force_n_mode	= 0;

	info->packets_to_be_sent= 0;
	info->receiver_finished	= 0;
	info->expected_mseq	= 0;
	info->no_timeout	= 1;
	info->recvbuf		= 0;
	info->size_of_packet	= 0;
	info->recv_window	= 1000000000;
	info->rto		= 3000000;
	return 0;
}


int send_packet(socket_udp *socket, struct shared *info, struct pkt_dtg *packet,  char* destinationfile, int size_of_last_packet,  FILE *file_pointer, double mulTFRC ) {
	int ret;
	packet->send_pkt= htonl(info->send_pkt);
	packet->mseq		= htonl(info->packets_sent);
	packet->dseq		= 0; /* unused */
	packet->ts		= htonl(get_time() / MS_TO_US);
	packet->rtt		= htonl(info->rtt_cur);
	packet->bitrate		= htonl((u_int32) info->b_act);
	packet->round		= htonl(info->round);
	packet->mode		= htonl(info->mode);

	packet->rto		= htonl(info->rto);
	packet->data_size	= htonl(DATA_SIZE);
	packet->timestamp	= htonl(get_time());
	packet->end_flag	= 0;
	packet->start_flag 	= 0;
if(mulTFRC==0){
	packet->mulTFRC         = htonl((u_int32)0);
//printf("mulTFRC %u \n", ntohl(packet->mulTFRC));
} else
	packet->mulTFRC		= htonl((u_int32_t)(mulTFRC*100));
	if((info->packets_sent +1) >= info->packets_to_be_sent) {
		packet->end_flag=1;
		packet->data_size = htonl(size_of_last_packet);
	}
	if(info->packets_sent == 0) {
		packet->start_flag=1;
		strncpy(packet->data, destinationfile, DATA_SIZE);
		packet->data_size = htonl(strlen(destinationfile));
	}
	else {
	       if((info->packets_sent +1) >= info->packets_to_be_sent) {
		fseek (file_pointer , ((info->packets_sent-1) * DATA_SIZE) , SEEK_SET);
		fread(packet->data, 1, size_of_last_packet, file_pointer);
//                strncpy(packet->data,data+((info->packets_sent-1) * DATA_SIZE),size_of_last_packet);
		}
		
		else{
			fseek (file_pointer , ((info->packets_sent-1) * DATA_SIZE) , SEEK_SET);
	                fread(packet->data, 1, DATA_SIZE, file_pointer);
		//strncpy(packet->data,data+((info->packets_sent-1) * DATA_SIZE),DATA_SIZE); /* reinitialize */	
		}
	}
	while ((ret = udp_send(socket, (char *) packet, PKT_SIZE)) < 0) {
		fprintf(stderr, "%3.2f ", (float) get_time() / MICROSEC);
		perror("udp_send");
	}
info->send_pkt+=1;
	return ret;
}


int send_loop(socket_udp *socket, struct shared *info, int cbr, char* sourcefile, char* destinationfile, double mulTFRC) {
	int buflen = PKT_SIZE;
	char buffer[buflen];
	struct pkt_dtg *packet = (struct pkt_dtg *) buffer;
	int64 start = 0, end = 0;
	int64 delay = 0, delay_diff = 0;
	int nr = 0;
	double modified_bact = 0;
	
	int size_of_last_packet = 0;
	FILE *file_pointer;
	file_pointer=fopen(sourcefile, "rb");		/* binary reading */
	
	/* getting filesize */
	long file_size;
	fseek (file_pointer , 0 , SEEK_END);
	file_size = ftell (file_pointer);
	rewind (file_pointer);
	printf("Filesize: %lu  \n", file_size);


	/* reading file data */
/*	char* file_data = (char*) malloc (sizeof(char)*file_size);;
	fread(file_data, 1, file_size, file_pointer);
	fclose(file_pointer);	
*/


	llnode * node, * tail, * head, *tmp;	/* pointer for linked list */
	head = NULL;				/* first element in the linked list */
	tail = NULL;				/* last element in the linked list */

	double diff = 0;			/* difference between actual time and sending time including RTT */
	
	info->packets_to_be_sent = (u_int32)ceil((double)file_size/DATA_SIZE)+1;

	size_of_last_packet = (int)file_size%DATA_SIZE;
	if(size_of_last_packet == 0) {
		size_of_last_packet = DATA_SIZE;
	}

	while ((info->receiver_finished == 0)){//&&(nr < MAX_TOTAL_PACKETS)) {
 start = get_time();
 		
		if(head != NULL) {

/*			 retransmission timer */
			diff=(double)head->time-(double)get_time()+(double)(info->rto);
 
			if((diff < 0.0) && (info->no_timeout == 0)) {


				//printf("timeout while waiting for packet %lu\n",info->expected_mseq);
/*				 no further timeout can occurr until new packet is sent */
				info->no_timeout = 1;		
				info->packets_sent = info->expected_mseq;
/*				 restart receiving window */

				if(info->recvbuf > 0) { 
					info->recv_window = info->recvbuf;
				}
				else { 
/*					 prevent deadlock */
					info->recv_window = 1000000000;
				}

			}
/*			 linked list management 	*/
/*			 remove acknowledged packets from the linked list */


			while(head != tail) {
				if(head->packet_id < info->expected_mseq) {    
				
					tmp = head->next;
					free(head);
					head = tmp;
				}
				else {

					break;
				}

/*				else {*/
/*					if(head == NULL) {*/
/*						*/
						/* list already empty */
/*					}*/
/*					else {*/
/*						//free(head);*/
/*					}*/
/*					tail = NULL;*/
/*					break;*/
/*				}*/
				
			}
		}
		
		
		if((info->recv_window >= info->size_of_packet) && (info->packets_sent < info->packets_to_be_sent)){


			if (cbr == 0) { /* dynamic congestion control according to TFRC specification */
				switch (info->mode) {
					case M_INITRTT:
						break;
					case M_SLOWSTART:
						/* inc by delta over one rtt */
						if (start > info->adj_ts + info->rtt * MS_TO_US) {
							if (info->b_rep > 0) {
								info->b_exp = min(2 * info->b_rep, 2 * info->b_act);
							}
							if (info->b_exp > info->max_bitrate)
								info->b_exp = info->max_bitrate;
							/* use of b_exp for interval is conservative */
							assert(info->b_exp != 0);
							assert(info->rtt != 0);
							info->rate_delta = (info->b_exp - info->b_act);

							/* and spread increase over packets sent during the next RTT */
							info->rate_delta /= max((info->b_act + info->rate_delta / 2) / PKT_SIZE * (info->rtt / MILLISEC), 1);

							if (info->rate_delta < 0.0) 
								info->rate_delta = 0.0;
							else
								info->adj_ts = start;
						}
						if (info->b_act + info->rate_delta < info->b_exp)
							info->b_act += info->rate_delta;
						else
							info->b_act = info->b_exp;
						break;
						assert(info->b_act != 0);

					case M_NORMAL:
						/* inc smoothly each time a packet is sent */
						info->b_act = min(info->b_act + info->rate_delta, info->max_increase);
						assert(info->b_act != 0);
						break;
					default:
						fprintf(stderr, "error: invalid mode (%i)\n", info->mode);
						break;
				}
			} else { /* no congestion control, possible for UDP comparison */
				info->b_act = info->b_exp = cbr;
			}

			assert(info->b_act != 0);

			++nr;
			info->size_of_packet = send_packet(socket, info, packet, destinationfile, size_of_last_packet, file_pointer,  mulTFRC);
			info->recv_window = info->recv_window - info->size_of_packet; 
			/* ... and wait (to send at specified bitrate) */
			end = get_time();

			info->no_timeout = 0;			/* timeout can occurr after packet is sent */
	
			/* adding new value to link list */
			/* check to remove values from linked list in case of retransmission */
			if(info->packets_sent <= info->expected_mseq) {
				while (head != tail) {    
					tmp = head->next;
					free(head);
					head = tmp;
				}
				if(head != NULL)
					free(head);
				tail = NULL;
			}
			/* addition of time into the linked list*/
			node = (llnode *)malloc(sizeof(llnode));
			node->time = end;
			node->packet_id = info->packets_sent;
			if(tail == NULL) {
				tail = node;
			}
			else {
				tail->next = node;
				tail = node;
			}
			if(head == NULL) {
				head = node;
			}					

			++info->packets_sent; 			/* sequence number, starting with 0 */
			
			/* modify sending rate, should not influence Internet experiments but improve Dummynet behavior */
			if (info->rtt_cur != 0 && info->mode == M_NORMAL) {
				modified_bact = info->b_act * info->sqrt_rtt / sqrt(info->rtt_cur);
				/* printf("%6.3f: %3.2f \t %3.2f \t %3.2f \t %3.2f\n", (double) start / MICROSEC, */
					/* info->b_act, modified_bact, info->sqrt_rtt, sqrt(info->rtt_cur)); */
			} else {
				modified_bact = info->b_act;


			}


			delay  = (int64) PKT_SIZE * MICROSEC / modified_bact; 	/* the time it should take */


			delay -= (end - start);                               	/* minus the time it took */
			delay += delay_diff;                                  	/* plus diff of last packet */

			wait(delay);	

			delay_diff = delay - (get_time() - end); 		/* waited too long / too short */

		} /* end recv_window check*/
	} /* end sending loop */
	/*if(head != NULL) {
		printf("ID of head: %lu\n", head->packet_id);
	}*/
	while(head != tail) {
		//printf("ID of packet: %lu\n", head->packet_id);
		tmp = head;
		head = head->next;
		free(tmp);
	}
	if(head != NULL) {
		free(head);
	}
	//printf("Cleanup complete\n");
//	if(tail != NULL)
//		printf("ID of tail: %lu\n", tail->packet_id);
//	printf("Packages sent: %u info->receiver_finished: %d\n", nr, info->receiver_finished );
//	printf("Receiving window: %u \n", info->recv_window);
//	printf("Packet size: %u \n", info->size_of_packet);
//	printf("Received acks: %lu\n", info->reports_recv);
//	free(file_data);
	fclose(file_pointer);
	if (nr == MAX_TOTAL_PACKETS)
		fprintf(stderr, "maximum number of packets (%i) reached\n", MAX_TOTAL_PACKETS);
	return 0;
}


void update_mode(struct shared *info, double b_exp) {
	switch (info->mode) {
	case M_INITRTT:
		if (info->rtt > 0.0) {
			info->mode = M_SLOWSTART;
			info->b_act = PKT_SIZE / (info->rtt / MILLISEC) / 2;
			info->b_rep = PKT_SIZE / (info->rtt / MILLISEC) / 2;
			//printf("%f slowstart mode\n", (float) info->rep_ts / MILLISEC);
		}
		break;
	case M_SLOWSTART:
		if (b_exp > 0) {
			info->mode = M_NORMAL;
			info->adj_ts = 0;
			//printf("%f congestion control mode\n", (float) info->rep_ts / MILLISEC);
		}
		break;
	case M_NORMAL:
		/* back to SLOWSTART mode when b_exp == 0 (that's when l_exp == 0) */
		if (b_exp == 0 && info->force_n_mode == 0) {
			info->b_exp = 0;
			info->mode = M_SLOWSTART;
			info->adj_ts = 0;
			//printf("%f reentered slowstart mode\n", (float) info->rep_ts / MILLISEC);
		}
		break;
	default:
		fprintf(stderr, "error: invalid mode (%i)\n", info->mode);
		exit(1);
	}
}


void update_bexp(struct shared *info, double b_exp) {
	if (info->mode == M_NORMAL && b_exp > 0) {
		info->b_exp = (double) b_exp;
		info->b_exp = max(info->b_exp, info->min_bitrate);
		info->b_exp = min(info->b_exp, info->max_bitrate);

		//if (info->b_act >= info->b_exp) {
			info->b_act = info->b_exp;
			info->max_increase = info->b_exp;
			info->rate_delta = 0;
			assert(info->max_increase > 0);
/*		} else {*/
			/* inc by 1 packet */
/*			info->rate_delta  = PKT_SIZE / (info->rtt / MILLISEC);*/
			/* limit total increase to 2 packets/rtt until new rec. reports arrive */
/*			info->max_increase = min(info->b_exp, info->b_act + 2 * info->rate_delta);*/
/*min*/
			/* and spread increase over the approx. number of packets sent during the next RTT */
/*			info->rate_delta /= (info->b_act + info->rate_delta / 2) / PKT_SIZE * (info->rtt / MILLISEC);*/
/*			assert(info->max_increase > 0);*/
/*		}*/
		/* printf("%f: %f + %f -> %f\n", (float) info->rep_ts / MILLISEC, */
		/* info->b_act, info->rate_delta, info->b_exp);  */
	}
}


int recv_reports(socket_udp *socket, struct shared *info) {
	char buffer[REP_SIZE];
	struct rep_dtg *rep = (struct rep_dtg *) buffer;
	struct timeval max_wait_tv;
	double max_wait;
	int nready;

	u_int32 ack_mseq;	/* acknowledged sequence number via receiver report */
	int counter;		/* counter to check for duplicate ACKS */
	int64 actual;		/* actual time */

	double granularity = 1000 * CLOCK_GRANULARITY; 	/* clock granularity */
	double r = 0;
	double srtt = 0;
	double rttvar = 0;
	float alpha = 8;	/* values suggested in Jacobson, V. and M. Karels, "Congestion Avoidance and Control"*/
	float beta = 4;

	int number_acknowledged_packets;	/* number of acknowledged packets with one receiver report */

	int64 time_of_last_ack = get_time();
	int64 max_wait_for_ack = MAX_WAIT * 1000000;
	int64 time_of_last_rate_change = get_time();

	udp_fd_zero();
	while (info->receiver_finished == 0) {
	
		/* detection of timeouts */
		actual = get_time();
		
		/* check for program termination */
		if(actual > time_of_last_ack + max_wait_for_ack) {
			info->receiver_finished=1;
			//printf("No acknowledgement received for %u seconds, process will be terminated \n", MAX_WAIT);
		}

		/* TFRC part */
		udp_fd_set(socket);

		/* managing TFRC variables for transmission rate */
		switch (info->mode) {
		case M_INITRTT:
			/* assume 500ms RTT for timeout while in M_INITRTT */
			max_wait = DEC_NO_REPORT * 500.0 / MILLISEC / REPORTS_PER_RTT;
			break;
		case M_SLOWSTART:
			/* use SLOWSTART_NO_REPORT instead of DEC_NO_REPORT, be more cautious */
			max_wait = (double) SLOWSTART_NO_REPORT * 
				max(max(info->rtt_cur, info->rtt) / MILLISEC / REPORTS_PER_RTT,
					PKT_SIZE / info->b_act);
			break;
		case M_NORMAL:
			/* use max (RTT_CUR, RTT, time to send one packet)
				-> avoid decrease caused by heavy RTT variations */
			max_wait = (double) DEC_NO_REPORT * 
				max(max(info->rtt_cur, info->rtt) / MILLISEC / REPORTS_PER_RTT,
					PKT_SIZE / info->b_act);
			break;
		default:
			fprintf(stderr, "error: invalid mode (%i)\n", info->mode);
			exit(1);
		}

		max_wait_tv.tv_sec = (long) max_wait;
		max_wait_tv.tv_usec = ((long) (max_wait * MICROSEC)) % MICROSEC;


/*		if ((actual/MICROSEC) > ((time_of_last_rate_change/MICROSEC) + max_wait)) {*/
		if ((nready = udp_select(&max_wait_tv)) <= 0) {
			nready = 0;
			time_of_last_rate_change = get_time();
			if (nready == 0) {
				/* no receiver report for DEC_NO_REPORT x "expected time interval" 
					* -> assume congestion and reduce rate */
				/*printf("%f timeout: no report for %3.2fms\n", (float) info->rep_ts / MILLISEC,
					max_wait * MILLISEC);*/

				if (info->mode == M_SLOWSTART) {
					info->mode = M_NORMAL;
					/* set adj_ts to 0 so that rate_delta is reset to an appropriate value
						as soon as we get out of slowstart */
					info->adj_ts = 0;
					/* keep sending at the current rate */
					info->b_exp = info->b_act;
					info->rate_delta = 0;
					info->max_increase = info->b_act;
					//printf("%f congestion control mode\n", (float) info->rep_ts / MILLISEC);
				} else if (info->mode == M_NORMAL) {
					info->b_act = max(info->b_act / 2, info->min_bitrate);
					info->max_increase = info->b_act;
					info->rate_delta = 0;
					info->b_exp = info->b_act;
				}

				assert(info->max_increase > 0);
				assert(info->b_act > 0);

				++info->round; /* sender reacted by halving bitrate -> start new round */
				continue; /* check here */
			} else if (errno == EINTR) {
				continue;
			} else {
				perror("select_error");
			}
		}

		/* receiving reports and acknowledgements */
		if (udp_fd_isset(socket)) {

			if (udp_recv(socket, buffer, REP_SIZE)) {

				/* getting the size of the buffer at the receiver */
				if(info->recvbuf == 0) {
					info->recvbuf = ntohl(rep->recvbuf);
					//printf("Receiving buffer: %u\n", info->recvbuf);
					info->recv_window = info->recvbuf - (info->packets_sent - info->expected_mseq) * info->size_of_packet;
				}
				
				/* managing TFRC variables */
				info->rep_ts = get_time() / MS_TO_US;
				info->rtt_cur = info->rep_ts - ntohl(rep->ts);
				++info->round; /* new report -> start new round for loss measurement */
				if (info->rtt > 0.0)
					info->rtt = (1 - RTT_DECAY) * info->rtt + RTT_DECAY * info->rtt_cur;
				else
					info->rtt = info->rtt_cur;

				if (info->sqrt_rtt > 0.0)
					info->sqrt_rtt = (1 - RTT_DECAY) * info->sqrt_rtt + RTT_DECAY * sqrt(info->rtt_cur);
				else
					info->sqrt_rtt = sqrt(info->rtt_cur);

				++info->reports_recv;

				//r=get_time()-ntohl((double)rep->timestamp);
				r=(get_time()-ntohl((double)rep->timestamp))/MICROSEC;				

				if(srtt == 0.0) {
					srtt = r;
					rttvar = r/2;
				}
				else {
					rttvar = (1 - (1/beta)) * rttvar + (1/beta) * abs(srtt-r);
					srtt = (1 - (1/alpha)) * srtt + (1/alpha) * r;
				}
				//info->rto = max(1000000, srtt + max(granularity, 4*rttvar));
				info->rto = MICROSEC * max(0.2, srtt + max(granularity / MICROSEC, 4*rttvar));
				
				

				/* store time to check how much time has passed since last arrival of packet */
				time_of_last_ack = get_time();
				time_of_last_rate_change = time_of_last_ack;

				/* get the acknowledged sequence number */
				ack_mseq    = ntohl(rep->mseq);



				/* check if ack for final packet has been received */
				if((ack_mseq + 1) >= info->packets_to_be_sent) {
					info->receiver_finished=1;
					printf("Last ACK received at %f, transfer finished\n",  (double)actual/1000000);
				}
				/* check for retransmission of acknowledgement */
				if(ack_mseq == (info->expected_mseq-1)) {	/* ACK for last packet received again - double ACK */
					if(ntohl((int)rep->ignore_packet)==1) {
						/* retransmission only used to update values */
					}
					else {
						++counter;
					}
				}
				if(ack_mseq >= info->expected_mseq) {		/* new in order packet */					
					counter = 1;
 					/* check for number of acknowledged packets */
					number_acknowledged_packets = ack_mseq - info->expected_mseq;
					info->recv_window = (number_acknowledged_packets + 1) * info->size_of_packet + info->recv_window;
					info->expected_mseq = ack_mseq+1;
				}
			//	if(ack_mseq==100) {
			//		printf("Speed: %f\n", (double)ntohl(rep->b_exp));
			//	}
				info->b_rep = ntohl(rep->b_rep);
				update_mode(info, ntohl(rep->b_exp));
				update_bexp(info, ntohl(rep->b_exp));

				if(counter >= 2){	/* double ACK has been received */
					info->packets_sent = ack_mseq + 1;
					counter = 0;
					//printf("duplicate ACK received \n");
				}
			}
		}
	}
	return 0;
}


int main(int argc, char *argv[]) {
	char *addr = NULL;
	char *sourcefile = NULL;
	char *destinationfile = NULL;
	int port = -1;
	int ttl = 16;
	int pid_recv;
	socket_udp *send_socket;
	socket_udp *recv_socket;
	char   *shmptr;
	int    shmid;
	struct shmid_ds buf;
	struct shared *info;
	//int    duration = 0;
	int    cbr = 0;
	double max_bitrate = MAX_BITRATE;
	double min_bitrate = 0;
	int i;
	double mulTFRC = 0;
	if (argc < 5) {
		fprintf(stderr, "usage: %s address port sourcefile destinationfile [ttl] [-m mulTFRC] [-y max. bitrate] [-i min. bitrate] [-c const. bitrate]\n", argv[0]);
		exit(1);
	} else {
		addr = argv[1];
		port = atoi(argv[2]);
		sourcefile = argv[3];
		destinationfile = argv[4];
		for (i = 5; i < argc; ++i) {
			if (argv[i][0] != '-') {
				ttl = atoi(argv[i]);
			} else {
				switch (argv[i][1]) {
				case 'm' : if (argc > i + 1) mulTFRC = atof(argv[++i]); break;
				case 'y' : if (argc > i + 1) max_bitrate = atof(argv[++i]); break;
				case 'i' : if (argc > i + 1) min_bitrate = atof(argv[++i]); break;
				case 'c' : if (argc > i + 1) cbr = atoi(argv[++i]); break; /* UDP equivalent, no congestion control */
				default : fprintf(stderr, "unknown option: %s\n", argv[i]); exit(1); break;
				}
			}
		}
	}

	printf("started ");
	for (i = 0; i < argc; ++i)
		printf("%s ", argv[i]);
	printf("\n");

	send_socket = udp_init(addr, port  , ttl); /* initiate socket for sending */
	recv_socket = udp_init(addr, port+1, ttl); /* initiate socket for receiving, stated port number +1 */
	assert(send_socket != NULL);
	assert(recv_socket != NULL);

	shmid = shmget (IPC_PRIVATE, sizeof(struct shared), SHM_R | SHM_W) ; /* allocates a shared memory segment */
	if (shmid == -1) {
		perror("shmget");
		exit(1);
	}
	shmptr = shmat(shmid,0,0);
	if (shmptr == NULL) {
		perror("shmat");
		exit(1);
	}

	info = (struct shared *) shmptr; /* shared struct, used by sending process, manipulated by receiving process */

	shm_init(info, min_bitrate, max_bitrate);
	if (shmid == -1) {
		perror("shm_init");
		exit(1);
	}
	shmctl(shmid, IPC_RMID, &buf);

	init_time();

	pid_recv = fork (); /* one sending process, one receiving process */
	if (pid_recv == 0) {
		recv_reports(recv_socket, info);
	} else {
		send_loop(send_socket, info, cbr, sourcefile, destinationfile, mulTFRC);
		kill(pid_recv, SIGKILL); /* kill receiving process after sending loop is finished */
	}
	shmdt(shmptr);
//	printf("sender finished\n");

	exit(0);
}
